﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraZoom : MonoBehaviour {

    GameController mapConfig;
    float mouseWheel;
    float zHeight;
    float maxAngle;
    float spanningWidth;
    float spanningHeight;

    public float edgeScrollingPixels = 50f;
    public float lerpSpeed = 55f;
    public float minZoom = -3f;
    public float zoomSpeed = 0.5f;

    private float startTimeX;
    private float startTimeY;
    private float startTimeZoomOut;
    

    // Use this for initialization
    void Start () {
        mapConfig = GameObject.FindGameObjectWithTag("Controller").GetComponent<GameController>();
        spanningHeight = (CubeCell.height * mapConfig.row) * 0.75f - CubeCell.height * 0.25f; //the height that the camera can see (top - bottom)
        spanningWidth = (mapConfig.col * CubeCell.width);
        float longestEdge = spanningHeight > spanningWidth ? spanningHeight : spanningWidth;
        float hypotenuse = Mathf.Sqrt(spanningHeight * spanningHeight + spanningWidth * spanningWidth);
        zHeight = -hypotenuse*Mathf.Tan(Mathf.Deg2Rad * (90 - GetComponent<Camera>().fieldOfView));
        transform.position = new Vector3(spanningWidth/2, spanningHeight/2, zHeight);
        maxAngle = Camera.main.fieldOfView;
        startTimeX = Time.time;
        startTimeY = Time.time;
    }
	
	// Update is called once per frame
	void Update () {

        Vector2 mouse = Input.mousePosition;
        if(mouse.x < edgeScrollingPixels) {
            if(Vector3.Angle(new Vector3(1, 0, 0), new Vector3(transform.position.x, 0, transform.position.z)) <= maxAngle) {
                float distCovered = (Time.time - startTimeX);
                float fracJourney = distCovered / lerpSpeed;
                transform.position = Vector3.Lerp(transform.position, new Vector3(-CubeCell.width / 2, transform.position.y, transform.position.z), fracJourney);
            }
        }
        else if(mouse.x > (Screen.width - edgeScrollingPixels)) {
            if (Vector3.Angle(new Vector3(1, 0, 0), new Vector3(spanningWidth - transform.position.x, 0, transform.position.z)) <= maxAngle) {
                float distCovered = (Time.time - startTimeX);
                float fracJourney = distCovered / lerpSpeed;
                transform.position = Vector3.Lerp(transform.position, new Vector3(spanningWidth, transform.position.y, transform.position.z), fracJourney);
            }
        }
        else
            startTimeX = Time.time;

        if (mouse.y < edgeScrollingPixels) {
            if (Vector3.Angle(new Vector3(0, transform.position.y, transform.position.z), new Vector3(0, 1, 0)) <= maxAngle) {
                float distCovered = (Time.time - startTimeY);
                float fracJourney = distCovered / lerpSpeed;
                transform.position = Vector3.Lerp(transform.position, new Vector3(transform.position.x, -CubeCell.height / 2, transform.position.z), fracJourney);
            }
        }
        else if(mouse.y > (Screen.height - edgeScrollingPixels)) {
            if (Vector3.Angle(new Vector3(0, 1, 0), new Vector3(0, spanningHeight - transform.position.y, transform.position.z)) <= maxAngle) {
                float distCovered = (Time.time - startTimeY);
                float fracJourney = distCovered / lerpSpeed;
                transform.position = Vector3.Lerp(transform.position, new Vector3(transform.position.x, spanningHeight, transform.position.z), fracJourney);
            }
        }
        else
            startTimeY = Time.time;

        //Debug.Log(Vector3.Angle(-transform.position, new Vector3(-CubeCell.width / 2, -CubeCell.height / 2, 0)));
        mouseWheel = Input.GetAxis("Mouse ScrollWheel");


        // Allows zooming in and out via the mouse wheel
        if (mouseWheel < 0)
        {
            if (transform.position.z > zHeight)
            /*{
                float distCovered = (Time.time - startTimeZoomOut);
                float fracJourney = distCovered / lerpSpeed;
                Vector3.Lerp(transform.position, new Vector3(spanningWidth / 2, spanningHeight / 2, zHeight), fracJourney);
            }*/
                transform.Translate(0, 0, -zoomSpeed, Space.World);
            //transform.position = new Vector3(transform.position.x, transform.position.y, Mathf.Clamp(transform.position.z, -3, -100));
            //Debug.Log("(" + transform.position.x + ", " + transform.position.y + ", " + transform.position.z + ")");
        }
        if (mouseWheel > 0)
        {
            if (transform.position.z < minZoom)
                transform.Translate(0, 0, zoomSpeed, Space.World);
            //transform.position = new Vector3(transform.position.x, transform.position.y, Mathf.Clamp(transform.position.z, -3, -100));
            //transform.position = new Vector3(hit.point.x, hit.point.y, Mathf.Clamp(transform.position.z, -6, -10));
            //Debug.Log("(" + transform.position.x + ", " + transform.position.y + ", " + transform.position.z + ")");
        }
        else
            startTimeZoomOut = Time.time;
    }
}
