﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum TileType
{
    WATER,
    PLAINS,
    HILL,
    FOREST,
    MOUNTAIN,
    RUINS,
    BASE,
};

public class CubeCell{

    protected int x;
    protected int y;
    protected int z;
    protected TileType tile;
    public static float height = 0.64f;
    public static float width = 0.64f;

    public CubeCell(int x, int y, int z, TileType t)
    {
        this.x = x;
        this.y = y;
        this.z = z;
        this.tile = t;
    }

    public CubeCell(Vector3 v, TileType t)
    {
        this.x = (int)v.x;
        this.y = (int)v.y;
        this.z = (int)v.z;
        this.tile = t;
    }

    public TileType getTile()
    {
        return tile;
    }

    public int getX()
    {
        return x;
    }

    public int getY()
    {
        return y;
    }

    public int getZ()
    {
        return z;
    }

    public Vector3 getCube()
    {
        return new Vector3(x, y, z);
    }

    public Vector3 getAxial()
    {
        return cubeToAxial(getCube());
    }

    public void setTile(TileType t)
    {
        tile = t;
    }

    static public Vector3 rectToHex(Vector3 rect)
    {
        return cubeToAxial(roundHex(axialToCube(new Vector3((rect.x * Mathf.Sqrt(3) / 3 - rect.y / 3) / height*2, rect.y * 2 / 3 / width*2,0))));
    }

    static public Vector3 axialToCube(Vector3 ax) {
        float x = ax.x;
        float z = ax.y;
        return new Vector3(x, -x - z, z);
    }

    static public Vector3 cubeToAxial(Vector3 cube)
    {
        return new Vector3(cube.x, cube.z, 0);
    }

    static private Vector3 roundHex(Vector3 cube)
    {
        int q = (int)(Mathf.Round(cube.x));
        int r = (int)(Mathf.Round(cube.y));
        int s = (int)(Mathf.Round(cube.z));
        double q_diff = Mathf.Abs(q - cube.x);
        double r_diff = Mathf.Abs(r - cube.y);
        double s_diff = Mathf.Abs(s - cube.z);
        if (q_diff > r_diff && q_diff > s_diff)
        {
            q = -r - s;
        }
        else
            if (r_diff > s_diff)
        {
            r = -q - s;
        }
        else
        {
            s = -q - r;
        }
        return new Vector3(q, r, s);
    }

}
