﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Map {
    CubeCell[,] array;
    private int row;
    private int col;

	public Map(int row, int col)
    {
        array = new CubeCell[row, col];
        this.row = row;
        this.col = col;
        genMap();

    }
    private void setAxial(Vector3 axial, CubeCell cell)
    {
        //Debug.Log(axial);
        Vector2Int pos = axialToArray(axial);
        //Debug.Log(pos);
        array[pos.x,pos.y] = cell;
    }
    public CubeCell getAxial(int row, int col)
    {
        return array[row, col + row / 2];
    }

    public CubeCell getRect(int row, int col)
    {
        return array[row, col];
    }

    private Vector2Int axialToArray(Vector3 axial)
    {
        return new Vector2Int((int)axial.x + ((int)axial.y) / 2, (int)axial.y);
    }

    private void genMap()
    {
        for (int i = 0; i < row; i++) {
            for(int j = 0; j< col; j++) {
                TileType type;
                if(i > 1 && i < row -2 && j > 1 && j < col -2)
                {
                    float r = Random.Range(0f,1f);
                    if (r < .25)
                    {
                        type = TileType.MOUNTAIN;
                    }
                    else
                    {
                        if(r<.65)
                        {
                            type = TileType.PLAINS;
                        }
                        else
                        {
                            if (r<.15)
                            {
                                type = TileType.HILL;
                            }
                            else
                            {
                                type = TileType.FOREST;
                            }
                        }
                    }

                    
                }
                else
                {
                    type = TileType.WATER;
                }
                
                array[i, j] = new CubeCell(i,-i -j, j, type);
                //Debug.Log((new Vector3(i, -i - j, j)));
            }
        }
        //minSmooth();
        //maxSmooth();
        smooth();
        placeBase();

    }

    private void placeBase()
    {
        int x = Random.Range(row/4, 3*row/4);
        int y = Random.Range(col/4, 3*col/4);
        BaseCell mainBase = new BaseCell(x, x - y / 2, -2 * x + y / 2);
        mainBase.setActive(true);
        array[x, y] = mainBase;
        int dX = Random.Range(3, 7);
        int dY = Random.Range(3, 7);
        if (Random.value < .5f)
        {
            dX = -dX;
        }
        if (Random.value < .5f)
        {
            dY = -dY;
        }
        Vector3 pos = mainBase.getAxial();
        //Debug.Log(pos);
        pos = new Vector3(pos.x + dX, pos.y + dY);
        BaseCell sideBase = new BaseCell(CubeCell.axialToCube(pos));
        //setAxial(pos, sideBase);
    }
    private void maxSmooth()
    {
        for (int i = 2; i < row - 2; i++)
        {
            for (int j = 2; j < col - 2; j++)
            {
                int axJ = j - i / 2;
                int max = 0;
                CubeCell[] nbors = getAxialNeighbors(i, axJ);
                for (int n = 0; n < 7; n++)
                {
                    int tile = (int)nbors[n].getTile();
                    if (tile > max)
                    {
                        max = tile;
                    }
                }
                array[i, j].setTile((TileType)max);
            }
        }
    }

    private void minSmooth()
    {
        for (int i = 3; i < row - 3; i++)
        {
            for (int j = 3; j < col - 3; j++)
            {
                int axJ = j - i / 2;
                int min = 100;
                CubeCell[] nbors = getAxialNeighbors(i, axJ);
                for (int n = 0; n < 7; n++)
                {
                    int tile = (int)nbors[n].getTile();
                    if (tile < min)
                    {
                        min = tile;
                    }
                }
                array[i, j].setTile((TileType)min);
            }
        }
    }

    private void smooth()
    {
        for (int i = 2; i < row - 2; i++)
        {
            for (int j = 2; j < col - 2; j++)
            {
                int axJ = j - i / 2;
                float sum = 0;
                CubeCell[] nbors = getAxialNeighbors(i, axJ);
                for (int n = 0; n < 7; n++)
                {
                    sum += (float)nbors[n].getTile();
                }
                int avg = (int)Mathf.Round(sum / 7);
                array[i, j].setTile((TileType)avg);
            }
        }
    }

    private CubeCell[] getAxialNeighbors(int q, int r)  {
        CubeCell[] nbors = new CubeCell[7];
        nbors[0] = getAxial(q, r);
        nbors[1] = getAxial(q-1, r);
        nbors[2] = getAxial(q+1, r);
        nbors[3] = getAxial(q, r-1);
        nbors[4] = getAxial(q, r+1);
        nbors[5] = getAxial(q+1, r-1);
        nbors[6] = getAxial(q-1, r+1);
        return nbors;
        }
}
